# xtuml.__xtuml_lextab.py. This file automatically created by PLY (version 3.4). Don't edit!
_tabversion   = '3.2'
_lextokens    = {'NUMBER': 1, 'PHRASE': 1, 'GUID': 1, 'MINUS': 1, 'INSERT': 1, 'RPAREN': 1, 'SEMICOLON': 1, 'CREATE': 1, 'REF_ID': 1, 'TO': 1, 'COMMA': 1, 'FRACTION': 1, 'ROP': 1, 'TABLE': 1, 'STRING': 1, 'INTO': 1, 'LPAREN': 1, 'ID': 1, 'RELID': 1, 'FROM': 1, 'VALUES': 1, 'CARDINALITY': 1}
_lexreflags   = 0
_lexliterals  = ''
_lexstateinfo = {'INITIAL': 'inclusive'}
_lexstatere   = {'INITIAL': [('(?P<t_comment>\\-\\-([^\\n]*\\n))|(?P<t_COMMA>,)|(?P<t_FRACTION>(\\d+)(\\.\\d+))|(?P<t_RELID>R[0-9]+)|(?P<t_CARDINALITY>(1C))|(?P<t_ID>[A-Za-z_][\\w_]*)|(?P<t_LPAREN>\\()|(?P<t_MINUS>-)|(?P<t_NUMBER>[0-9]+)|(?P<t_RPAREN>\\))|(?P<t_SEMICOLON>;)|(?P<t_STRING>\\\'((\\\'\\\')|[^\\\'])*\\\')|(?P<t_GUID>\\"([^\\\\\\n]|(\\\\.))*?\\")|(?P<t_newline>\\n+)', [None, ('t_comment', 'comment'), None, ('t_COMMA', 'COMMA'), ('t_FRACTION', 'FRACTION'), None, None, ('t_RELID', 'RELID'), ('t_CARDINALITY', 'CARDINALITY'), None, ('t_ID', 'ID'), ('t_LPAREN', 'LPAREN'), ('t_MINUS', 'MINUS'), ('t_NUMBER', 'NUMBER'), ('t_RPAREN', 'RPAREN'), ('t_SEMICOLON', 'SEMICOLON'), ('t_STRING', 'STRING'), None, None, ('t_GUID', 'GUID'), None, None, ('t_newline', 'newline')])]}
_lexstateignore = {'INITIAL': ' \t\r\x0c'}
_lexstateerrorf = {'INITIAL': 't_error'}
